#!/usr/bin/python

#^#
# OVSCDS.py
#
# CDS -- Concurrent Data Store, a term borrowed from berkeley db.
# but in fact, in our case, the db access is serialized. 
#
# 
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#

from time import sleep
from os.path import dirname, join
from bsddb import dbshelve
from OVSConstants import CONFIG_DIR
from OVSTimer import Timer
from OVSFileLock import FileLock

class Timeout(Exception):
    def __init__(self, msg):
	Exception.__init__(self, msg)

class CDS(object): 
    #Concurrent Data Store
    #FIXME: here we use a simple file lock to serialize db access
    # make sure close the CDS as soon as possible 
    # & avoid any deadlock, for e.g. donnot raise Exception before close()
    def __init__(self, *db_names):
        self.db_names = list(db_names)
	#avoid deadlock
        self.db_names.sort()
        db_base = join(CONFIG_DIR, "db" )
	self.lock_timeout = False

        #lock db
        self.lock = {}
	lck_idx = 0
	try:
	    for (idx, db_name) in enumerate(self.db_names):
		lock_file = join(db_base, "%s.lock" % (db_name,) )
		self.lock[db_name] = FileLock(lock_file)
		#if cannot acquire a lock in 10 sec, timeout
		timer = Timer(30, 0.1)
		while timer.wait():
		    if self.lock[db_name].acquire():
			break
		if timer.timeout(): 
		    lck_idx = idx
		    raise Timeout("accquire lock timeout!")
		db_file = join(db_base, "%s.db" % db_name)
		dbs = dbshelve.open(db_file) 
		setattr(self, "db_%s" % db_name, dbs)
        except Timeout, e:
	    for idx in range(0, lck_idx + 1):
		self.lock[db_names[idx]].release()
	    raise e
        self._opened = True #sentinel

    def close(self):
        if self._opened: #so that call close() multitimes is ok in a process
            self._opened = False
            for db_name in self.db_names:
                try:
                    try:
                        getattr(self, "db_%s" % db_name).close()
                        delattr(self, "db_%s" % db_name)
                    finally:
                        self.lock[db_name].release()
                except: pass

if __name__ == "__main__":
    pass
