#!/usr/bin/env python

#^#
# OVSCommons.py
#
# common function used in agent.
#
# 
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#
import os
import popen2
from OVSConfig import get_config
from os.path import abspath

def parse_result(result):
    """
    all return result following same convension, (our own marshal mechanism):
    success                         #if success completed without return value
    success:key=value;key=value...  #if success completed with return result
    failed:errmsg                   #failed with error message 
    """
    if result.startswith("failed"): return

    #e.g. result="success:key=value;key2=value2..."
    d = {}
    result = result[8:].split(";")
    if len(result) == 1 and result[0].strip() == "":
	return d
    for kv in result:
        k,v = kv.split("=")
        d[k] = v
    return d

def host_alive(ip):
    """Determine whether the given host is alive."""
    status = os.popen("ping -q -c 2 %s" % ip).close()
    return not status

def fmt_err_msg(e):
    return "<%s: %s>" % (e.__class__.__name__, e)

def run_cmd(cmd):
    """
    run a cmd and catch its output, if run into any problem raise exception
    """
    #N.B.: !!security checkpoint!! 
    #possible cmd injection, like sql injection
    p = popen2.Popen3(cmd, True)
    if p.wait() != 0:
	raise Exception('%s=>%s' % (cmd, p.childerr.read()))
    return p.fromchild.read()
    
def chk_vm_path(vm_path):
    """
    check path agaist vm operation
    """
    vm_path = abspath(vm_path)
    config = get_config()
    paths = config.get("security", "vm.store").split(":")
    for path in paths:
	if vm_path.startswith(path.strip()): return
    raise Exception("'%s' is not inside vm store!" % vm_path)

def exposed(func):
    #---------------------------------------
    def wrapper(*args, **kwargs):
        return func(*args, **kwargs)
    #---------------------------------------
    wrapper.exposed = True
    wrapper.real_func = func
    return wrapper

