#!/usr/bin/env python
#^#
# OVSSiteVMConfig.py
#
# <description>
#
# 
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#
import logging
from os.path import join, basename
from OVSSiteUtility import get_vm_size, get_vdisk_size
from OVSServerSelection import select_one_server, get_utility_server
from OVSDB import db_get_vm, db_get_real_vm_path
from OVSCommons import exposed, parse_result, fmt_err_msg, chk_vm_path
from OVSXUtility import find_real_path

@exposed
def add_vm_disk(vm_path, disk_path, sharable=False):
    """
    attach a disk to a vm

    @return	    "success" if success else "failed:<error message>"	
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(vm_path)
	chk_vm_path(disk_path)

	vm = db_get_vm(vm_path)
	#(hostname, sp) = select_one_server(vm["vtech"])
	(hostname, sp) = get_utility_server()
        real_vm_path = db_get_real_vm_path(vm_path)
        real_disk_path = find_real_path(disk_path)
        logger.debug("add_vm_disk:disk_path=%s, real_disk_path=%s" % (disk_path, real_disk_path))
	rs = getattr(sp, "%s_add_disk" % vm["vtech"])(real_vm_path, real_disk_path, sharable)
	
	logger.info("add_vm_disk: vm('%s') disk('%s'): rs=%s" % \
		(vm_path, disk_path, rs))
	return rs
    except Exception,e:
	errmsg = fmt_err_msg(e)
	logger.error("add_vm_disk: failed. vm('%s') disk('%s') => %s" % \
		(vm_path, disk_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def remove_vm_disk(vm_path, disk_path):
    """
    detach a disk from a vm

    @return	    "success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(vm_path)
	chk_vm_path(disk_path)

	vm = db_get_vm(vm_path)
	#(hostname, sp) = select_one_server(vm["vtech"])
	(hostname, sp) = get_utility_server()
        real_vm_path = db_get_real_vm_path(vm_path)
        real_disk_path = find_real_path(disk_path)
	rs = getattr(sp, "%s_remove_disk" % vm["vtech"])(real_vm_path, real_disk_path)
	
	logger.info("remove_vm_disk: vm('%s') disk('%s'): rs=%s" % (vm_path, disk_path, rs))
	return rs
    except Exception,e:
	errmsg = fmt_err_msg(e)
	logger.error("remove_vm_disk: failed. vm('%s') disk('%s') => %s" % \
		(vm_path, disk_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def set_vm_memory(vm_path, mem_size):
    """
    set vm memory size in the configuration file, restart vm to take effect.
    @param mem_size must > 256
    @return         "success:mem_size=<mem size>" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(vm_path)

	vm = db_get_vm(vm_path)
	#(hostname, sp) = select_one_server(vm["vtech"])
        try:
            mem_size = int(mem_size)
            if mem_size < 256:
                raise Exception("mem_size must > 256.")
        except ValueError:
            raise Exception("mem_size must be integer.")

	(hostname, sp) = get_utility_server()
        real_vm_path = db_get_real_vm_path(vm_path)
	rs = getattr(sp, "%s_set_memory" % vm["vtech"])(real_vm_path, mem_size)

	logger.info("set_vm_memory: vm('%s') mem_size=%d: rs=%s" % \
		(vm_path, mem_size, rs))
	return rs
    except Exception,e:
	errmsg = fmt_err_msg(e)
	logger.error("set_vm_memory: failed. vm('%s') mem_size=%d => %s" % \
		(vm_path, mem_size, errmsg))
	return "failed:%s" % errmsg

@exposed
def get_vm_memory(vm_path):
    """
    get vm memory size

    @return	    "success:mem_size=<mem size>" if success else "failed:<error message>"	
    """
    logger = logging.getLogger("ovs.query")
    try:
	vm = db_get_vm(vm_path)
	#(hostname, sp) = select_one_server(vm["vtech"])
	(hostname, sp) = get_utility_server()
        real_vm_path = db_get_real_vm_path(vm_path)
	rs = getattr(sp, "%s_get_memory" % vm["vtech"])(real_vm_path)

	logger.info("get_vm_memory: vm('%s') rs=%s" % \
		(vm_path, rs))
	return rs
    except Exception,e:
	errmsg = fmt_err_msg(e)
	logger.error("get_vm_memory: failed. vm('%s') => %s" % \
		(vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def set_vm_vcpus(vm_path, vcpus):
    """
    set vm cpu in the configuration file, restart vm to take effect.

    @return		"success" if success else "failed:<error message>"	
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(vm_path)

	vm = db_get_vm(vm_path)
	#(hostname, sp) = select_one_server(vm["vtech"])
	(hostname, sp) = get_utility_server()
        real_vm_path = db_get_real_vm_path(vm_path)
	rs = getattr(sp, "%s_set_vcpus" % vm["vtech"])(real_vm_path, vcpus)
	
	logger.info("set_vm_vcpus: vm('%s') vcpus=%d: rs=%s" % \
		(vm_path, vcpus, rs))
	return rs
    except Exception,e:
	errmsg = fmt_err_msg(e)
	logger.error("set_vm_vcpus: failed. vm('%s') vcpus=%d => %s" % \
		(vm_path, vcpus, errmsg))
	return "failed:%s" % errmsg

@exposed
def get_vm_vcpus(vm_path):
    """
    get vm cpu

    @return	    "success:vcpus=<num of vcpus>" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.query")
    try:
	vm = db_get_vm(vm_path)
	#(hostname, sp) = select_one_server(vm["vtech"])
	(hostname, sp) = get_utility_server()
        real_vm_path = db_get_real_vm_path(vm_path)
	rs = getattr(sp, "%s_get_vcpus" % vm["vtech"])(real_vm_path)
	
	logger.info("get_vm_vcpus: vm('%s') rs=%s" % \
		(vm_path, rs))
	return rs
    except Exception,e:
	errmsg = fmt_err_msg(e)
	logger.error("get_vm_vcpus: failed. vm('%s') => %s" % \
		(vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def set_vm_vif(vm_path, vif):
    """
    set vm vif in the configuration file, restart vm to take effect.

    @return	    "success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(vm_path)

	vm = db_get_vm(vm_path)
	#(hostname, sp) = select_one_server(vm["vtech"])
	(_hostname, sp) = get_utility_server()
	from socket import gethostname
	hostname = gethostname() 
        real_vm_path = db_get_real_vm_path(vm_path)
	rs = getattr(sp, "%s_set_vif_num" % vm["vtech"])(real_vm_path, vif, hostname)
	
	logger.info("set_vm_vif: vm('%s') vif=%d: rs=%s" % \
		(vm_path, vif, rs))
	return rs
    except Exception,e:
	errmsg = fmt_err_msg(e)
	logger.error("set_vm_vif: failed. vm('%s') vif=%d => %s" % \
		(vm_path, vif, errmsg))
	return "failed:%s" % errmsg

@exposed
def get_vm_vif(vm_path):
    """
    get vm vif

    @return	    "success:vif=<num of vif>" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.query")
    try:
	vm = db_get_vm(vm_path)
	#(hostname, sp) = select_one_server(vm["vtech"])
	(hostname, sp) = get_utility_server()
        real_vm_path = db_get_real_vm_path(vm_path)
	rs = getattr(sp, "%s_get_vif_num" % vm["vtech"])(real_vm_path)
	
	logger.info("get_vm_vif: vm('%s') rs=%s" % \
		(vm_path, rs))
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("get_vm_vif: failed. vm('%s') => %s" % \
		(vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def set_vm_vifs(vm_path, vif_cfg, is_append=False):
    """
    set vm vif in the configuration file, restart vm to take effect.
    @param vif_cfg      refer to xen_set_vifs()
    @param is_append    refer to xen_set_vifs()
    @return	    "success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(vm_path)

	(_hostname, sp) = get_utility_server()
        real_vm_path = db_get_real_vm_path(vm_path)
	vm = db_get_vm(vm_path)
	rs = getattr(sp, "%s_set_vifs" % vm["vtech"])(real_vm_path, vif_cfg, is_append)
	
	logger.info("set_vm_vifs: vm('%s') vif_cfg=%s => rs=%s" % \
		(vm_path, vif_cfg, rs))
	return rs
    except Exception,e:
	errmsg = fmt_err_msg(e)
	logger.error("set_vm_vifs: failed. vm('%s') vif_cfg=%s => %s" % \
		(vm_path, vif_cfg, errmsg))
	return "failed:%s" % errmsg

@exposed
def get_vm_vifs(vm_path):
    """
    get vm vif in the configuration file, 
    @return      refer to xen_get_vifs()
    """
    logger = logging.getLogger("ovs.query")
    try:
	chk_vm_path(vm_path)

	(_hostname, sp) = get_utility_server()
        real_vm_path = db_get_real_vm_path(vm_path)
	vm = db_get_vm(vm_path)
	rs = getattr(sp, "%s_get_vifs" % vm["vtech"])(real_vm_path)
	
	logger.info("get_vm_vifs: vm('%s') => rs=%s" % (vm_path, rs))
	return rs
    except Exception,e:
	errmsg = fmt_err_msg(e)
	logger.error("get_vm_vifs: failed. vm('%s') => %s" % (vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def clear_vm_vif_mac(vm_path):
    """
    wrapper for xen_clear_vif_mac()

    @return	    "success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(vm_path)

	(_hostname, sp) = get_utility_server()
        real_vm_path = db_get_real_vm_path(vm_path)
	vm = db_get_vm(vm_path)
	rs = getattr(sp, "%s_clear_vif_mac" % vm["vtech"])(real_vm_path)

	logger.info("clear_vm_vif_mac: vm('%s') => rs=%s" % (vm_path, rs))
	return rs
    except Exception,e:
	errmsg = fmt_err_msg(e)
	logger.error("clear_vm_vif_mac: failed. vm('%s') => %s" % (vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def get_vm_vdisks(vm_path):
    """
    get vm vdisks

    @return	    "success:vdisks=<string of vdisks>" if success else "failed:<error message>"
    examples of string of vdisks: "system.img:500,vmlinuz:10,initrd.gz:2", "system.raw:500"
    500, 10, 2 are the sizes of the files and block size is 1M.
    """
    logger = logging.getLogger("ovs.query")
    try:
	chk_vm_path(vm_path)
	vm = db_get_vm(vm_path)
	#(hostname, sp) = select_one_server(vm["vtech"])
	(hostname, sp) = get_utility_server()
	
        real_vm_path = db_get_real_vm_path(vm_path)
	vm_cfg_file = join(real_vm_path, "vm.cfg")
	rs = getattr(sp, "%s_correct_cfg" % vm["vtech"])(vm_cfg_file, real_vm_path)
	if not rs.startswith("success"): 
            raise Exception(rs)

	rs = getattr(sp, "%s_get_vdisks" % vm["vtech"])(real_vm_path)
	if not rs.startswith("success"): 
            raise Exception(rs)
	vdisks_str = parse_result(rs)["vdisks"]

	#when vdisks not exist
	if vdisks_str == "":
	    rs = ""
	else:
	    vdisks = vdisks_str.split(",")
	    result = []
	    for vdisk in vdisks:
		rs_size = get_vdisk_size(vdisk)
		if not rs_size.startswith("success"): 
                    raise Exception(rs_size)
		size = parse_result(rs_size)["size"]
		result.append("%s:%s" % (vdisk, size))
	    rs = ','.join(result)

	logger.info("get_vm_vdisks: success. vm('%s') rs=%s" % (vm_path, rs))
	return "success:vdisks=%s" % rs
    except Exception,e:
	errmsg = fmt_err_msg(e)
	logger.error("get_vm_vdisks: failed. vm('%s') => %s" % (vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def get_vm_cfg_vdisks(vm_path):
    """
    get vm vdisks from the cfg file 
    N.B.: 
        the original reason to implement this function is that, 
        when wget a vm, user want to first check if disks assosiated with this vm available.

    @return	"success:vdisks=<string of vdisks>" if success else "failed:<error message>"
    example of string of vdisks: "system.img,vmlinuz,initrd.gz"
    """
    logger = logging.getLogger("ovs.query")
    try:
	chk_vm_path(vm_path)
	vm = db_get_vm(vm_path)
	#(hostname, sp) = select_one_server(vm["vtech"])
	(hostname, sp) = get_utility_server()
	
        real_vm_path = db_get_real_vm_path(vm_path)
	rs = getattr(sp, "%s_get_vdisks" % vm["vtech"])(real_vm_path)
	if not rs.startswith("success"): raise Exception(rs)
	vdisks_str = parse_result(rs)["vdisks"]

	if not vdisks_str:
	    rs = "success:vdisks="
	    logger.info("get_vm_cfg_vdisks: success. vm_path('%s') => %s" % \
		    (vm_path, rs))
	    return rs

	vdisks = vdisks_str.split(",")
	vdisks_lst = []
	for vdisk in vdisks:
	    vdisk = basename(vdisk)
	    vdisks_lst.append(vdisk)
	vdisks_str = ",".join(vdisks_lst)

	rs = "success:vdisks=%s" % vdisks_str
	logger.info("get_vm_cfg_vdisks: success. vm_path('%s') => %s" % \
		(vm_path, rs))
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("get_vm_cfg_vdisks: failed. vm_path('%s') => %s" % \
		(vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def get_vm_memcpudisksize(vm_path):
    """
    get vm memory, vcpus, vdisks and vm size

    @return
    "success:mem_size=<mem size>;vcpus=<num of vcpus>;vdisks=<string of vdisks>;vm_size=<vm size>" if success
    else "failed:<error message>"
    examples of string of vdisks: "system.img:500,vmlinuz:10,initrd.gz:2", "system.raw:500"
    """
    logger = logging.getLogger("ovs.query")
    try:
        real_vm_path = db_get_real_vm_path(vm_path)

	rs_mem = get_vm_memory(real_vm_path)
	if not rs_mem.startswith('success:'): raise Exception(rs_mem)
	mem = int(parse_result(rs_mem)['mem_size'])

	rs_cpu = get_vm_vcpus(real_vm_path)
	if not rs_cpu.startswith('success:'): raise Exception(rs_cpu)
	cpu = int(parse_result(rs_cpu)['vcpus'])

	rs_disk = get_vm_vdisks(real_vm_path)
	if not rs_disk.startswith('success:'): raise Exception(rs_disk)
	disk = parse_result(rs_disk)['vdisks']

	rs_size = get_vm_size(real_vm_path)
	if not rs_size.startswith('success:'): raise Exception(rs_size)
	size = int(parse_result(rs_size)['size'])

	rs = "success:mem_size=%d;vcpus=%d;vdisks=%s;vm_size=%d" % (mem, cpu, disk, size)

	logger.info("get_vm_memcpudisksize:success. vm('%s') => %s", (vm_path, rs))
	return rs

    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("get_vm_memcpudisksize:failed. vm('%s') => %s" % \
		(vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def get_vm_type(vm_path):
    """
    get vm type

    @return	"success:type=<type>" if success else "failed:<error message>"
    type should be "hvm" or "para"
    """
    logger = logging.getLogger("ovs.query")
    try:
	chk_vm_path(vm_path)
	vm = db_get_vm(vm_path)
	(hostname, sp) = select_one_server(vm["vtech"])
	logger.debug("get_vm_type: utility server '%s' is selected." % hostname)
	
        real_vm_path = db_get_real_vm_path(vm_path)
	rs = getattr(sp, "%s_get_vm_type" % vm["vtech"])(real_vm_path)
	if not rs.startswith("success"): raise Exception(rs)
	
	logger.info("get_vm_type: success. vm_path('%s') => %s" % (vm_path, rs))
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("get_vm_type: failed. vm_path('%s') => %s" % (vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def get_vm_vnc_password(vm_path):
    """
    get vm vnc password from the cfg file

    @return	"success:vnc_password=<vnc_password>" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.query")
    try:
	chk_vm_path(vm_path)
	vm = db_get_vm(vm_path)
	(hostname, sp) = get_utility_server()
	
        real_vm_path = db_get_real_vm_path(vm_path)
	rs = getattr(sp, "%s_get_vm_vnc_password" % vm["vtech"])(real_vm_path)
	if not rs.startswith("success"): raise Exception(rs)
	
	logger.info("get_vm_vnc_password: success. vm_path('%s') => %s" % (vm_path, rs))
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("get_vm_vnc_password: failed. vm_path('%s') => %s" % (vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def set_vm_vnc_password(vm_path, vnc_password):
    """
    set vm vnc password

    @return	"success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(vm_path)
	vm = db_get_vm(vm_path)
	(hostname, sp) = select_one_server(vm["vtech"])
	
        real_vm_path = db_get_real_vm_path(vm_path)
	rs = getattr(sp, "%s_set_vm_vnc_password" % vm["vtech"])(real_vm_path, vnc_password)
	if not rs.startswith("success"): raise Exception(rs)
	
	logger.info("set_vm_vnc_password: success. vm_path('%s') => %s" % \
		(vm_path, rs))
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("set_vm_vnc_password: failed. vm_path('%s'), => %s" % \
		(vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def add_vm_vif(vm_path, num):
    """
    add vifs for the vm

    @return	"success:macs=<macs_string>" if success else "failed:<error message>"
    macs_string is in the format of "mac1,mac2"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(vm_path)
	vm = db_get_vm(vm_path)
        real_vm_path = db_get_real_vm_path(vm_path)

	(_hostname, sp) = select_one_server(vm["vtech"])
	
	from socket import gethostname
	hostname = gethostname()
	rs = getattr(sp, "%s_add_vif" % vm["vtech"])(real_vm_path, num, hostname)
	if not rs.startswith("success"): raise Exception(rs)
	
	logger.info("add_vm_vif: success. vm_path('%s'), num('%s') => %s" % \
		(vm_path, num, rs))
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("add_vm_vif: failed. vm_path('%s'), num('%s') => %s" % \
		(vm_path, num, errmsg))
	return "failed:%s" % errmsg

@exposed
def del_vm_vif(vm_path, macs):
    """
    del vifs

    @param macs	    macs is a string in the format of "mac1,mac2"
    @return	"success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(vm_path)
	vm = db_get_vm(vm_path)
        real_vm_path = db_get_real_vm_path(vm_path)
	(hostname, sp) = select_one_server(vm["vtech"])
	
	rs = getattr(sp, "%s_del_vif" % vm["vtech"])(real_vm_path, macs)
	if not rs.startswith("success"): raise Exception(rs)
	
	logger.info("del_vm_vif: success. vm_path('%s'), macs('%s') => %s" % \
		(vm_path, macs, rs))
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("del_vm_vif: failed. vm_path('%s'), macs('%s') => %s" % \
		(vm_path, macs, errmsg))
	return "failed:%s" % errmsg

