#!/usr/bin/env python
#^#
# OVSSiteVMInstall.py
#
# <description>
#
# 
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#

import logging
from OVSSiteXenStore import get_vm_vnc_conn
from OVSSiteDB import register_vm 
from OVSServerSelection import get_utility_server, select_one_server, select_tgt_host, get_vm_host
from OVSDB import db_update_vm, db_get_vm, db_get_real_vm_path
from OVSCommons import exposed, fmt_err_msg, chk_vm_path
from OVSConstants import OVS_BASE
from OVSXUtility import get_storage_repos, assemble_real_path, find_real_path

#---------------------<> vm installation <>----
@exposed
def install_vm_hvm(vm_path, memory, vcpus, vif_cfg, disks, cdrom, vncpassword, vnclisten="0.0.0.0"):
    """
    install full virtualized vm from scratch
    @param memory	size in MB
    @param vcpus	cpu number
    @param vif_cfg      refter to xen_set_vifs(), and another constrain is 
                        mac in this param follow all or none convention, i.e.
                        "<mac>,<bridge>;<mac>,<bridge>" or ",<bridge>;,<bridge>"
    @param disks	disk string, in format "disk_path:disk_size:is_sparse" e.g.
			system.raw:2:False
		    disk path should be relative to vm_path
		    disk_size is counted in GB
		    is_sparse should be True or False
    @param cdrom	path to the install media iso file
    @return		"success:vnc=<vnc server>:<vnc port>" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(vm_path)

	disks_list = disks.split(",")
	disks_list = [disk.split(":") for disk in disks_list]
	vm_size = 0
	for disk_path, disk_size, sparse_option in disks_list:
	    vm_size += float(disk_size)

	(repos, free_space) = get_storage_repos()
	if free_space < vm_size + 10: 
	    raise Exception("not enough free space")
	real_vm_path = assemble_real_path(vm_path, repos)

	vtech = "xen"
	register_vm(vm_path, vtech)
	(hostname, sp) = select_tgt_host(vm_path, any=True)
	db_update_vm(vm_path, { "host_srv" : hostname, "real_path":real_vm_path })

        real_cdrom = find_real_path(cdrom, True)[0]
	disks = disks.replace(vm_path, real_vm_path)
	rs = getattr(sp, "%s_install_vm_hvm" % vtech)\
                (real_vm_path, int(memory), int(vcpus), vif_cfg, disks, real_cdrom, vncpassword, vnclisten)

	if rs.startswith("success"):
	    logger.info("install_vm_hvm: vm('%s') => %s" % (vm_path, rs))
	    db_update_vm(vm_path, {"status": "RUNNING"})
	else: raise Exception(rs)

	return "success"
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("install_vm_hvm: vm('%s')=>%s " % (vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def change_vm_cdrom(vm_path, iso_path):
    """
    @return "success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(vm_path)
        real_iso_path = find_real_path(iso_path, True)[0]
	vm = db_get_vm(vm_path)
	(hostname, sp) = get_vm_host(vm_path)
        real_vm_path = db_get_real_vm_path(vm_path)
	rs = getattr(sp, "%s_change_vm_cdrom" % vm["vtech"])\
                (real_vm_path, real_iso_path)

	logger.info("change_vm_cdrom:vm('%s') iso('%s') => %s" % \
		(vm_path, iso_path, rs))
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("change_vm_cdrom:error. vm('%s') iso('%s') => %s " % \
		(vm_path, iso_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def install_vm_para(vm_path, memory, vcpus, vif_cfg, disks, location, vncpassword, vnclisten="0.0.0.0"):
    """
    install para virtualized vm from scratch
    @param memory	size in MB
    @param vcpus	cpu number
    @param vif_cfg      refer to install_vm_hvm()
    @param disks	disk string, in format "disk_path:disk_size:is_sparse" e.g.
			system.raw:2:False
		    disk path should be relative to vm_path
		    disk_size is counted in GB
		    is_sparse should be True or False
    @param location	location of the install directory e.g.
			nfs:host:path
			http and ftp protocols are also supported e.g.
			http://host/path
			ftp://host/path
    @return		"success:vnc=<vnc server>:<vnc port>" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(vm_path)

	disks_list = disks.split(",")
	disks_list = [disk.split(":") for disk in disks_list]
	vm_size = 0
	for disk_path, disk_size, sparse_option in disks_list:
	    vm_size += float(disk_size)
	
	(repos, free_space) = get_storage_repos()
	if free_space < vm_size:
	    raise Exception("not enough free space")
	real_vm_path = assemble_real_path(vm_path, repos)

	vtech = "xen"
	register_vm(vm_path, vtech)
	(hostname, sp) = select_tgt_host(vm_path, any=True)
	db_update_vm(vm_path, {"host_srv" : hostname, "real_path": real_vm_path})

	disks = disks.replace(vm_path, real_vm_path)
	rs = getattr(sp, "%s_install_vm_para" % vtech)\
                (real_vm_path, int(memory), int(vcpus), vif_cfg, disks, location, vncpassword, vnclisten)
	if rs.startswith("success"):
	    logger.info("install_vm_para: vm('%s') => %s" % (vm_path, rs))
	    db_update_vm(vm_path, {"status": "RUNNING"})
	else: raise Exception(rs)

	return "success"
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("install_vm_para: vm('%s')=>%s" % (vm_path, errmsg))
	return "failed:%s" % errmsg

