#!/usr/bin/env python
#^#
# OVSSiteXenStore.py
#
# <description>
#
# 
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#
import logging
from OVSDB import db_get_vm, db_get_srv, db_get_real_vm_path
from OVSCommons import exposed, parse_result, fmt_err_msg
from OVSServerProxy import ServerProxy

@exposed
def get_vm_vnc_conn(vm_path):
    """
    get vm console vnc port
    @return	    "success:vnc=<server:port>" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.query")
    try:
	vm = db_get_vm(vm_path)
	srv = db_get_srv(vm['host_srv'])
	sp = ServerProxy(srv['agt_url'])
	
	real_vm_path = db_get_real_vm_path(vm_path)
	rs = getattr(sp, "%s_get_vnc_port" % vm['vtech'])(real_vm_path)
	if rs.startswith("success"):
	    d = parse_result(rs)
	    vnc_port = d['vnc_port']
	else: 
	    raise Exception(rs)

	logger.info("get_vm_vnc_conn:success. vm('%s') => %s:%s" \
		% (vm_path, vm["host_srv"], vnc_port))
	rs = "success:vnc=%s:%s" % (vm["host_srv"], vnc_port)
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("get_vm_vnc_conn:failed. vm('%s') => %s" \
		% (vm_path, errmsg))
	return 'failed:%s' % errmsg

@exposed
def get_vm_vnc_port(hostname, vm_path, vtech='xen'):
    """
    get vm vnc port
    @return	    "success:vnc_port=<vnc_port>" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.query")
    try:
	srv = db_get_srv(hostname)
	if not vtech in srv["srv_role"]:
	    raise Exception("%s is not a %s server" % (hostname, vtech))
	sp = ServerProxy(srv['agt_url'])
	
	real_vm_path = db_get_real_vm_path(vm_path)
	rs = getattr(sp, "%s_get_vnc_port" % vtech)(real_vm_path)
	if rs.startswith("success"):
	    d = parse_result(rs)
	    vnc_port = d["vnc_port"]
	else:
	    raise Exception(rs)

	logger.info("get_vm_vnc_port:success. vm('%s') => %s:%s" \
		% (vm_path, hostname, vnc_port))
	rs = "success:vnc_port=%s" % vnc_port
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("get_vm_vnc_port:failed. vm('%s'), host('%s') => %s" \
		% (vm_path, hostname, errmsg))
	return 'failed:%s' % errmsg

@exposed
def get_network_bridges(hostname, vtech="xen"):
    """
    @param hostname     server you want to get the bridges info
    @return <refer to xen_get_network_bridges()>
    """
    logger = logging.getLogger("ovs.query")
    try:
        srv = db_get_srv(hostname)
        sp = ServerProxy(srv["agt_url"])
        rs = getattr(sp, "%s_get_network_bridges" % vtech)()
        if not rs.startswith("success"): 
            raise Exception(rs)
        logger.info("get_network_bridges: srv('%s') => %s" % (hostname, rs))
        return rs
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("get_network_bridges:failed. srv('%s') => %s" % (hostname, errmsg))
        return "failed:%s" % errmsg
