#!/usr/bin/env python
#^#
# OVSXXenStore.py
#
# <description>
#
# 
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#
import re
import logging
from os.path import basename, dirname, join, exists
from OVSXXenVMConfig import xen_unmarshall_cfg
from OVSConstants import OVS_CONFIG
from OVSCommons import parse_result, fmt_err_msg, run_cmd, exposed

#----------------------------<> helper function <>----
def xen_get_vm_id(vm_path):
    "get vm_id from xm list vm_name"
    vm_name = basename(vm_path)
    stdout = run_cmd("xm list %s" % vm_name)
    vm_id = stdout.splitlines()[-1].split()[1]
    return vm_id

#----------------------------<> exported xenstore related query function <>----
@exposed
def xen_status_vm(vm_path):
    """
    xm status vm (from xm list output)
    """
    vm_name = basename(vm_path)
    logger = logging.getLogger("ovs.operation")
    try:
	stdout = run_cmd("xm list %s" % vm_name)
	lines = stdout.splitlines()
	state = lines[-1].split()[-2]
	
	logger.info("xen_status_vm: success. vm('%s') => stat='%s'" % \
		(vm_path, state))
	return "success:state=%s" % state
    except Exception, e:
	logger.error("xen_status_vm: failed. vm('%s')=>%s" % \
		(vm_path, fmt_err_msg(e)))
	return "failed:%s" % fmt_err_msg(e)

@exposed
def xen_get_vm_names():
    """
    parse live vm names from xm list 

    @return 'success:vm_names=vm1,vm2,...' if success else 'failed:<error message>'
    """
    names = []
    logger = logging.getLogger("ovs.query")
    try:
	stdout = run_cmd("xm list")
	lines = stdout.splitlines()
	names = [line.split()[0] for line in lines[1:]]
	names.remove("Domain-0")
	rs = ','.join(names)
	
	logger.info("xen_get_vm_names: live vm names=%s" % rs)
	return "success:vm_names=%s" % rs
    except Exception, e:
        errmsg = fmt_err_msg(e)
	logger.error("xen_get_vm_names: failed. error message=%s" % errmsg)
	return "failed:%s" % errmsg

@exposed
def xen_get_vms_name_vncport_macs():
    """
    @return 'success:vm_info=vm1@5900@mac1^mac2,vm2@5901@mac,...' if success else 'failed:<error message>'
    """
    names = []
    logger = logging.getLogger("ovs.query")
    try:
	stdout = run_cmd("xenstore-ls /local/domain")
	lines = stdout.splitlines()

        ptn_id = re.compile(r"^[0-9]+")
        ptn_name = re.compile(r'^\s+name')
        ptn_vncport = re.compile(r'^\s+vnc-port')
        ptn_mac = re.compile(r'^\s+mac')

        l_name_port = []
        dom_id, dom_name, vncport, l_mac = "", "", "", []
	for line in lines:
            if ptn_id.match(line):
                if dom_id and dom_id != "0": #previous domain & ignore domain-0
                    l_name_port.append("%s@%s@%s" % (dom_name, vncport, "^".join(l_mac)))
                dom_name, vncport, l_mac = "", "", []
                dom_id = line.split("=")[0].strip() 
            else: 
                if ptn_name.match(line):
                    dom_name = line.split("=")[1].strip(" '\"")
                elif ptn_vncport.match(line):
                    vncport = line.split("=")[1].strip(" '\"")
                elif ptn_mac.match(line):
                    l_mac.append(line.split("=")[1].strip(" '\""))
        if dom_id and dom_id != "0": #last domain
            l_name_port.append("%s@%s@%s" % (dom_name, vncport, "^".join(l_mac)))
        
        rs = ",".join(l_name_port)
	logger.info("xen_get_vms_name_vncport_macs: rs=%s" % rs)
	return "success:vm_info=%s" % rs
    except Exception, e:
        errmsg = fmt_err_msg(e)
	logger.error("xen_get_vms_name_vncport_macs: failed. error message=%s" % errmsg)
	return "failed:%s" % errmsg

@exposed
def xen_get_vm_mac(vm_path):
    """
    parse mac from xm list, xenstore-ls
    """
    vm_name = basename(vm_path)
    logger = logging.getLogger("ovs.query")
    try:
	id = xen_get_vm_id(vm_path) 
	vifs = '/local/domain/%s/device/vif' % id
	#grep macs
	stdout = run_cmd('xenstore-ls %s' % vifs)
	lines = stdout.splitlines()
	macs = [line.split("=")[1].strip(' "') 
		for line in lines if line.find("mac =") != -1]
	mac_str = ",".join(macs)

	logger.info("xen_get_vm_mac: vm('%s') mac=%s" % (vm_path, mac_str))
	return "success:mac=%s" % mac_str
    except Exception, e:
	try:
	    vm_cfg_file = join(vm_path, "vm.cfg")
	    cfg = xen_unmarshall_cfg(vm_cfg_file)
	    macs = []
	    if "vif" in cfg:
		for vif in cfg["vif"]:
		     if "mac" in vif:
			mac = dict([kv.strip().split("=") for kv in vif.split(",")])["mac"] 
			macs.append(mac)
	    if macs: 
		mac_str = ",".join(macs)
		logger.info("xen_get_vm_mac: vm('%s') mac=%s" % (vm_path, mac_str))
		return "success:mac=%s" % (mac_str,)
	    else: 
		raise Exception("VM is not running or no mac found in the vm.cfg file.")
	except Exception, ee:
	    errmsg = fmt_err_msg(ee)
	    logger.error("xen_get_vm_mac: failed. vm('%s')=>%s" % (vm_path, errmsg))
	    return "failed:%s" % errmsg

@exposed
def xen_mem_free():
    "get free memory available for domainU from xend perspective"
    logger = logging.getLogger("ovs.query")
    try:
        #from xend view
	cmd = "xm info | grep -E '^free_memory'"
	stdout = run_cmd(cmd)
        mem_free_xen = int(stdout.split(":")[1].strip())
        #from domain-0 view
        cmd = "free -m | awk '/-\/\+ buffers\/cache/ {print $NF;}'"
        stdout = run_cmd(cmd)
        mem_free = int(stdout.strip())
        #use the bigger one
        if mem_free_xen > mem_free:
             mem_free = mem_free_xen

	rs = "success:mem_free=%d" % mem_free
	logger.info("xen_mem_free:%s" % rs)
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("xen_mem_free: failed. =>%s" % (errmsg, ))
	return "failed:%s" % errmsg

@exposed
def xen_get_vnc_port(vm_path):
    "get vm console vnc port"
    logger = logging.getLogger("ovs.query")
    try:
	vm_id = xen_get_vm_id(vm_path)
	cmd = "xenstore-read /local/domain/%s/console/vnc-port" % vm_id
	stdout = run_cmd(cmd)
	vnc_port = stdout.strip()

	logger.info("xen_get_vnc_port: success. vm('%s') vnc_port=%s" \
		% (vm_path, vnc_port))
	return "success:vnc_port=%s" % vnc_port
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("xen_get_vnc_port: failed. vm('%s') => %s" \
		% (vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def xen_get_vm_path(vm_name):
    """
    get vm path by vm name
    """
    logger = logging.getLogger("ovs.query")
    try:
	vm_id = xen_get_vm_id(vm_name)
	cmd = "xenstore-ls /local/domain/0/backend/vbd/%s" % vm_id
	stdout = run_cmd(cmd)
	lines = stdout.splitlines()

	pt = [line for line in lines 
		    if line.startswith(' params') or line.startswith(' type')]
	z = zip(pt[0::2], pt[1::2])
	d = dict(z)

	path = ''
	for p, t in d.items():
	    if t.split('=')[1].strip('" \n') == 'file':
		if ('shareddisk' not in p.lower()) and (vm_name in p):
		    path = dirname(p.split('=')[1].strip('" \n'))
		    break
	if path == '':
	    raise Exception('can not get vm path')
	
	logger.info("xen_get_vm_path: vm('%s') => %s" % (vm_name, path))
	return "success:path=%s" % path
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("xen_get_vm_path: failed! vm('%s') => %s" % (vm_name, errmsg))
	return "failed:%s" % errmsg

@exposed
def xen_top_info():
    """
    get domain-U's performance info from xentop cmd.
    @return success:vm_name=cpu_idle:mem_free:hd_avail;vm_name=...
    """
    logger = logging.getLogger("ovs.performance")
    try:
	stdout = run_cmd("xentop -b -i1")
#sameple output
#xentop -b -i1
#xentop - 15:20:41   Xen 3.1.0
#2 domains: 1 running, 1 blocked, 0 paused, 0 crashed, 0 dying, 0 shutdown 
#Mem: 2088380k total, 2087504k used, 876k free    CPUs: 1 @ 1729MHz
#      NAME  STATE   CPU(sec) CPU(%)     MEM(k) MEM(%)  MAXMEM(k) MAXMEM(%) VCPUS NETS NETTX(k) NETRX(k) VBDS   VBD_OO   VBD_RD   VBD_WR SSID
#  Domain-0 -----r       3683    0.0    1799168   86.2   no limit       n/a     1    4     1227    23524    0        0        0        0    0
#      twin --b---          8    0.0     262144   12.6     262144      12.6     1    1        0    18412    2        0      727      474    0
##
	lines = stdout.splitlines()
	domains = {}
	for n, line in enumerate(lines):
	    lines[n] = line.strip().replace("no limit", "nolimit")
	    if re.match(r"^\s*NAME", line):
		head_n = n
		head = line.split()
	for n in range(head_n + 1, len(lines)):
	    parts = lines[n].split()
	    domain_name = parts[0]
	    domains[domain_name] = dict(zip(head[1:], parts[1:]))
	del domains["Domain-0"]
	#logger.debug("xen_top_info: domains=%s" % repr(domains))
	rs = []
	for name in domains:
	    #FIXME:we cannot get free disk space for domainU from domain0
	    cpu_usage = domains[name]["CPU(%)"]
	    mem_size = "%d" % (int(domains[name]["MEM(k)"])/1024)
	    rs.append("%s=%s:%s:%s" % (name, cpu_usage, mem_size, "0"))
	rs = ";".join(rs)
	logger.info("xen_top_info:success =>%s" % rs)
	return "success:%s" % rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("xen_top_info: failed.=>%s" % errmsg)
	return "failed:%s" % errmsg

@exposed
def xen_get_caps():
    logger = logging.getLogger("ovs.query")
    try:
	stdout = run_cmd("xm info")
	lines = stdout.splitlines()
	for line in lines:
	    if line.startswith("xen_caps"):
		break
	caps = line.split(":")[1].strip()
	
	logger.info("xen_get_caps: success. caps => %s" % caps)
	return "success:caps=%s" % caps
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("xen_get_caps: failed. => %s" % errmsg)
	return "failed:%s" % errmsg

@exposed
def xen_get_network_bridges():
    """
    get network bridges, 
    @return     "success:<netdev>=<mac>;<netdev>=<mac>;..." if success else "failed:<error message>"
                if MGMNT_IF exists, it will be place as the first entry
                e.g.: success:xenbr0=00:15:60:C3:07:A1;xenbr1=00:15:60:C3:07:A2
    """
    logger = logging.getLogger("ovs.query")
    try:
        #stdout = run_cmd("ifconfig | sed -nre 's/^p([^ ]+).*/\1/p'")
        stdout = run_cmd("ifconfig | sed -r -e '/^ /d' -e '/^\s*$/d'")
        lines = stdout.splitlines()
        d = {}
        for line in lines:
            parts = line.split()
            netdev, mac = parts[0], parts[-1]
            d[netdev] = mac
        
        mgmnt_if = ""
        if exists(OVS_CONFIG):
            f = open(OVS_CONFIG)
            for line in f:
                if line.strip().startswith("MGMNT_IF"):
                    mgmnt_if = line.split("=")[1].strip()
                    break
            f.close()
        bm = [] #(bridge=mac, ...)
        for netdev in d:
            is_mgmnt_if = False
            if netdev.startswith("p"):  #peth0
                netdev = netdev[1:] #eth0
                if netdev == mgmnt_if: 
                    is_mgmnt_if = True
                if netdev in d:
                    vif_num = re.search("[0-9]+", netdev).group()   #0
                    bridge = "xenbr%s" % vif_num    #xenbr0
                    if bridge in d:
                        if is_mgmnt_if:
                            bm.insert(0, "%s=%s" % (bridge, d[netdev]))
                            logger.debug("xen_get_network_bridges:is_mgmnt_if=%s" % mgmnt_if)
                        else:
                            bm.append("%s=%s" % (bridge, d[netdev]))
        
        rs = ";".join(bm)

        logger.info("xen_get_network_bridges: success. %s" % rs)
        return "success:%s" % rs
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("xen_get_network_bridges: failed. %s" % errmsg)
        return "failed:%s" % errmsg

