#!/usr/bin/env python
#^#
# OVSXXenVMConfig.py
#
# <description>
#
# 
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#

import string
import re
import os
import logging
from shutil import copyfile
from random import randint, seed
from os.path import join, isdir, isfile, exists, dirname, basename, abspath, isabs
from OVSTimer import Timer
from OVSCommons import host_alive, parse_result, fmt_err_msg, run_cmd, chk_vm_path, exposed

#-----------------------------------<>helper function<>----
def xen_unmarshall_cfg(cfg_file):
    """
    read in vm cfg file and parse it to a directory
    """
    if not isfile(cfg_file): 
        raise Exception("vm config file ('%s') does not exist." % cfg_file)

    cfg = {}
    #FIXME: !!security!! and xen itself used execfile :(
    execfile(cfg_file, {}, cfg)
    return cfg

def xen_marshall_cfg(cfg, cfg_file):
    """
    write back vm cfg to config file
    """
    #-------------------------------------
    def _pretty_list(l):
	"""
	return repr(l) with each element on a new line
	"""
	if len(l) < 2: return repr(l)
	from StringIO import StringIO
	sio = StringIO()
	sio.write("[")
	for e in l:
	    sio.write("%s,\n" % repr(e))
	sio.write("]")
	s = sio.getvalue()
	sio.close()
	return s
    #-------------------------------------
    lines = []
    for k, v in cfg.items():
	if isinstance(v, list):
	    lines.append("%s = %s\n" % (k, _pretty_list(v)))
	elif repr(v).startswith("<"): #e.g. import os
	    continue
	else:
	    lines.append("%s = %s\n" % (k, repr(v)))
    lines.sort()

    f = open(cfg_file, "w")
    f.writelines(lines)
    f.close()

def xen_update_cfg_nics(vm_cfg_file, nics_enc):
    """	
    update config file, point all the path to vm_path
    !!depreciated!! config nic will be removed
    """     	
    cfg = xen_unmarshall_cfg(vm_cfg_file)
    if "extra" in cfg:
	#the length limit for cmdline is?
	cfg["extra"] = re.sub(r"\s*ifcfgs=[^ ]*", "", cfg["extra"])
	cfg["extra"] += " ifcfgs=%s" % (nics_enc,)
    else:
	cfg["extra"] = "ifcfgs=%s" % (nics_enc,)
    xen_marshall_cfg(cfg, vm_cfg_file)

#----------------------------------------<> <>-----
@exposed
def xen_correct_cfg(vm_cfg_file, vm_path):
    """	
    update config file, point all the path to vm_path
    """     
    logger = logging.getLogger("ovs.operation")
    try:
        #backup
	vm_cfg_file_orig = "%s.orig" % vm_cfg_file
	if not exists(vm_cfg_file_orig):
	    copyfile(vm_cfg_file, vm_cfg_file_orig)

	cfg = xen_unmarshall_cfg(vm_cfg_file)

	if "name" in cfg:
	    cfg["name"] = basename(vm_path)

	if "kernel" in cfg:
	    if exists(join(vm_path, basename(cfg["kernel"]))):
		cfg["kernel"] = join(vm_path, basename(cfg["kernel"]))

	if "ramdisk" in cfg:
	    cfg["ramdisk"] = join(vm_path, basename(cfg["ramdisk"]))

	if "disk" in cfg:
	    disks = []
	    cdrom_disk = ""
	    for disk in cfg["disk"]:
		#"backend-dev,frontend-dev,mode"
		backend_dev, frontend_dev, mode = disk.split(",")
		if "cdrom" in frontend_dev: #will deal with cdrom later
		    cdrom_disk = disk; continue
		if not backend_dev: #donnot know how to deal with it
		    continue
		dev_type, dev_id = backend_dev.rsplit(":", 1)
		if dev_type.strip() == "file":
		    disk_path_orig = dev_id
		    if "shareddisk" in disk_path_orig.lower(): #share disk
			disk_path = disk_path_orig
			if exists(disk_path): disks.append(disk)
		    else: #normal file based disk
			disk_path = join(vm_path, basename(disk_path_orig))
			disk = disk.replace(disk_path_orig, disk_path)
			if exists(disk_path): disks.append(disk)
		else: #other device => phy, tap:aio, etc
		    if exists(dev_id): disks.append(disk)
	    #deal with cdrom
	    try:
		backend_dev, frontend_dev, mode = cdrom_disk.split(",")
		dev_type, dev_id = backend_dev.split(":")
		if not exists(dev_id): 
		    raise Exception("Does not exist.")
		disks.append(disk)
	    except: #add default
		disks.append(",hdc:cdrom,r")
		    
	    cfg["disk"] = disks

        #bug#3853,3829, remove uuid, and let xend generate one for it
	if "uuid" in cfg: del cfg["uuid"]
	xen_marshall_cfg(cfg, vm_cfg_file)
	
	logger.info("xen_correct_cfg: success. vm('%s') => cfg_file('%s')" %\
		(vm_path, vm_cfg_file))
	return "success"
    except Exception, e:
	logger.error("xen_correct_cfg: failed. vm('%s'), cfg_file('%s') => %s" %\
		(vm_path, vm_cfg_file, fmt_err_msg(e)))
	return "failed:%s" % fmt_err_msg(e)

#----------------------------------------<> vm memory <>-----
@exposed
def xen_set_memory(vm_path, mem_size):
    """
    set vm memory
    """
    logger = logging.getLogger("ovs.operation")
    vm_cfg_file = join(vm_path, "vm.cfg")
    try:
	chk_vm_path(vm_path)

	cfg = xen_unmarshall_cfg(vm_cfg_file)
	cfg["memory"] = mem_size
	xen_marshall_cfg(cfg, vm_cfg_file)
	logger.info("xen_set_memory:vm('%s') => memory=%dM" % (vm_path, mem_size))
	return "success:mem_size=%d" % mem_size
    except Exception, e:
	logger.error("xen_set_memory:vm('%s')=>%s" % (vm_path, fmt_err_msg(e)))
	return "failed:%s" % fmt_err_msg(e)

@exposed
def xen_get_memory(vm_path):
    """
    get vm memory (in MB)
    """
    logger = logging.getLogger("ovs.query")
    vm_cfg_file = join(vm_path, "vm.cfg")
    try:
	chk_vm_path(vm_path)

	cfg = xen_unmarshall_cfg(vm_cfg_file)
	mem_size = int(cfg["memory"])

	logger.info("xen_get_memory:vm('%s') => memory=%dM" % (vm_path, mem_size))
	return "success:mem_size=%d" % mem_size
    except Exception, e:
	logger.error("xen_get_memory:vm('%s')=>%s" % (vm_path, fmt_err_msg(e)))
	return "failed:%s" % fmt_err_msg(e)

#----------------------------------------<> vm vcpus <>-----
@exposed
def xen_set_vcpus(vm_path, vcpus):
    """
    set VM vcpus
    """
    logger = logging.getLogger("ovs.operation")
    cfg_file = join(vm_path, "vm.cfg")
    try:
	chk_vm_path(vm_path)

	cfg = xen_unmarshall_cfg(cfg_file)
	cfg["vcpus"] = vcpus
	xen_marshall_cfg(cfg, cfg_file)

	logger.info("xen_set_vcpus:success. vm('%s')=>vcpus=%d" % (vm_path, vcpus))
	return "success"
    except Exception, e:
	logger.error("xen_set_vcpus:failed. vm('%s')=>%s" % (vm_path, fmt_err_msg(e)))
	return "failed:%s" % fmt_err_msg(e)

@exposed
def xen_get_vcpus(vm_path):
    """
    get vm vcpu number
    """
    logger = logging.getLogger("ovs.query")
    vm_cfg_file = join(vm_path, "vm.cfg")
    try:
	chk_vm_path(vm_path)

	cfg = xen_unmarshall_cfg(vm_cfg_file)
        try:
	    vcpus = int(cfg["vcpus"])
        except KeyError:
            vcpus = 1

	logger.info("xen_get_vcpus:vm('%s') => vcpus=%d" % (vm_path, vcpus))
	return "success:vcpus=%d" % vcpus
    except Exception, e:
	logger.error("xen_get_vcpus:vm('%s')=>%s" % (vm_path, fmt_err_msg(e)))
	return "failed:%s" % fmt_err_msg(e)

#----------------------------<> virtual disk <>----
@exposed
def xen_get_vdisks(vm_path):
    """
    get vm vdisks from config file
    @return 'success:vdisks=<path_to_file1>,<path_to_file2>' if success else 'failed:<error message>'
    """
    logger = logging.getLogger("ovs.query")
    vm_cfg_file = join(vm_path, "vm.cfg")
    try:
	chk_vm_path(vm_path)

	cfg = xen_unmarshall_cfg(vm_cfg_file)
	vdisks = cfg["disk"]

	files = []
	for vdisk in vdisks:
	    filename = vdisk.split(",")[0].split(":")[-1].strip()
            if filename: #possible ',hdc:cdrom,r' 
                files.append(filename)
	file_str = ','.join(files)
	
	logger.info("xen_get_vdisks:vm('%s') => vdisks=%s" % (vm_path, file_str))
	return "success:vdisks=%s" % file_str
    except Exception, e:
	logger.error("xen_get_vdisks:vm('%s') => %s" % (vm_path, fmt_err_msg(e)))
	return "failed:%s" % fmt_err_msg(e)

@exposed
def xen_add_disk(vm_path, disk_path, sharable=False):
    """
    add disk to vm config file
    """
    logger = logging.getLogger("ovs.operation")
    vm_cfg_file = join(vm_path, "vm.cfg")
    try:
	chk_vm_path(vm_path)
	chk_vm_path(disk_path)

	cfg = xen_unmarshall_cfg(vm_cfg_file)

	#check duplicate
	for disk in cfg["disk"]:
	    path = disk.split(",")[0].split(":")[-1] 
	    if disk_path == path:
		raise Exception("disk already attached.")
	#attach disk to vm
	frontend_devs = [ disk.split(",")[1].split(":")[0] 
		for disk in cfg["disk"] ]
	frontend_devs.sort()
	last_dev = re.sub(r"[0-9]+", "", frontend_devs[-1])
	sc = chr(ord(last_dev[-1]) + 1) #sc -> serial char
	frontend_dev = "%s%s" % (last_dev[:-1], sc)
	if frontend_dev == "hdc": #reserve hdc for cdrom
	    frontend_dev = "hdd"
	#use file io, tap:aio is not stable enough yet
	disk = "file:%s,%s,w" % (disk_path, frontend_dev)
	if sharable: 
	    disk = "%s!" % disk
	cfg["disk"].append(disk)

	xen_marshall_cfg(cfg, vm_cfg_file)

	logger.info("xen_add_disk: success. vm('%s') disk('%s')" % \
		(vm_path, disk_path))
	return "success"
    except Exception, e:
	logger.error("xen_add_disk: error. vm('%s') disk('%s')=>%s" % \
		(vm_path, disk_path, fmt_err_msg(e)))
	return "failed:%s" % fmt_err_msg(e)

@exposed
def xen_remove_disk(vm_path, disk_path):
    """
    remove disk from config file
    """
    logger = logging.getLogger("ovs.operation")
    vm_cfg_file = join(vm_path, "vm.cfg")
    try:
	chk_vm_path(vm_path)

	cfg = xen_unmarshall_cfg(vm_cfg_file)
	tgt_idx = -1
	for (idx, disk) in enumerate(cfg["disk"]):
	    path = disk.split(":")[-1].split(",")[0] 
	    if disk_path == path:
		tgt_idx = idx
                break
	if tgt_idx != -1:
	    del cfg["disk"][tgt_idx]
	    xen_marshall_cfg(cfg, vm_cfg_file)
	    logger.info("xen_remove_disk: success. vm('%s') disk('%s')." % \
		    (vm_path, disk_path))
	else:
	    logger.warning("xen_remove_disk: success. warning! disk('%s') does not exist in vm('%s') config." % \
		    (disk_path, vm_path, ))

	return "success"
    except Exception, e:
	logger.error("xen_remove_disk: error. vm('%s') disk('%s') => %s" % \
	    (vm_path, disk_path, fmt_err_msg(e)))
	return "failed:%s" % fmt_err_msg(e)

#----------------------------<> virtual network interface <>----
def gen_mac(id, seed_=""):
    seed("%s:%s" % (seed_, id))
    return "00:16:3E:%02X:%02X:%02X" % \
            (randint(0, 0x7f), randint(0, 0xff), randint(0, 0xff))

@exposed
def xen_set_vif_num(vm_path, vif_num, site_name=""):
    """
    set vif

    N.B.:
    !!deprecated!! use xen_set_vifs() instead
    """
    logger = logging.getLogger("ovs.operation")
    vm_cfg_file = join(vm_path, "vm.cfg")
    try:
	chk_vm_path(vm_path)
	vif_num = int(vif_num)
	
	if vif_num < 0:
	    raise Exception("invalid number of vifs")
	if vif_num > 8: 
	    #http://wiki.xensource.com/xenwiki/XenFaq#head-9896478cf65a16f43ab4fb066f74c0e0d67a16ac
	    raise Exception("cannot add more than 8 interfaces.")
	cfg = xen_unmarshall_cfg(vm_cfg_file)

        seed_ = "%s:%s" % (site_name, vm_path)
	cfg["vif"] = []
	for i in range(vif_num):
	    vif_new = "bridge=xenbr0,mac=%s" % gen_mac(i, seed_)
	    cfg["vif"].append(vif_new)
	
	xen_marshall_cfg(cfg, vm_cfg_file)

	logger.info("xen_set_vif_num: success. vm('%s') set vif=%s" % \
	    (vm_path, repr(cfg["vif"])))
	return "success"
    except Exception, e:
        errmsg = fmt_err_msg(e)
	logger.error("xen_set_vif_num: error. vm('%s') set vif=%d =>%s" % \
		(vm_path, vif_num, errmsg))
	return "failed:%s" % errmsg

@exposed
def xen_get_vif_num(vm_path):
    """
    get vif num
    @return "success:vif_num=<val>" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.query")
    try:
	chk_vm_path(vm_path)

	vm_cfg_file = join(vm_path, "vm.cfg")
	cfg = xen_unmarshall_cfg(vm_cfg_file)
	if "vif" in cfg: vif = len(cfg["vif"])
	else: vif = 0
	logger.info("xen_get_vif_num: success. vm('%s') get vif_num=%d" % (vm_path, vif))
	return "success:vif=%d" % vif
    except Exception, e:
        errmsg = fmt_err_msg(e)
	logger.error("xen_get_vif_num: failed. vm('%s') =>%s" % (vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def xen_del_vif(vm_path, macs):
    """
    del vif from the cfg file

    @param macs     mac address delimited by ',', 
                    if is "" then remove all vifs
    @return "success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(vm_path)
	vm_cfg_file = join(vm_path, "vm.cfg")
	cfg = xen_unmarshall_cfg(vm_cfg_file)
	
	if "vif" in cfg:
	    vifs = []
	    macs_lst = macs.split(",")
            if not macs_list: #clear vif entries if no mac specified
                cfg["vif"] = vifs
                logger.info("xen_del_vif: remove all vif for vm('%s')" % vm_path)
            
            for vif in cfg["vif"]:
                to_remove = False
                for mac in macs_lst:
                    if vif.upper().strip().find(mac.uppper().strip()) != -1:
                        to_remove = True
                        break
                if not to_remove: vifs.append(vif)
	    cfg["vif"] = vifs
	
	xen_marshall_cfg(cfg, vm_cfg_file)
	logger.info("xen_del_vif: success. vm_path('%s'), macs('%s')" % \
		(vm_path, macs))
	return "success"
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("xen_del_vif: failed. vm_path('%s'), macs('%s') => %s" % \
		(vm_path, macs, errmsg))
	return "failed:%s" % errmsg

@exposed
def xen_add_vif(vm_path, num, site_name=""):
    """
    add more vifs

    N.B.: 
        !!deprecated!! use xen_set_vifs() instead.
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(vm_path)

        num = int(num)
	if num < 0:
	    raise Exception("invalid number of vifs")

	vm_cfg_file = join(vm_path, "vm.cfg")
	cfg = xen_unmarshall_cfg(vm_cfg_file)

	macs = []
	added = 0
	if not "vif" in cfg:
	    xen_set_vif_num(vm_path, num)
	else:
	    rs = xen_get_vif_num(vm_path)
	    if not rs.startswith("success"):
		raise Exception(rs)
	    vif_num = parse_result(rs)["vif"]
	    vif_num = int(vif_num)
	    if vif_num + num > 8:
		raise Exception("too many vifs: 8 at most")
	
	    seed_= "%s:%s" % (site_name, vm_path)
	    vifs_str = " ".join(cfg["vif"])
	    for i in range(vif_num + num):
		if not added < num:
		    break
		mac = gen_mac(i, seed_)
		if not mac in vifs_str:
		    macs.append(mac)
                    #FIXME: should we clear these info?
		    vif_new = "bridge=xenbr0,mac=%s" % mac
		    cfg["vif"].append(vif_new)
		    added = added + 1

	    xen_marshall_cfg(cfg, vm_cfg_file)

	mac_str = ",".join(macs)
	logger.info("xen_add_vif: success. vm_path('%s') => %s vifs added" % \
		(vm_path, added))
	return "success:macs=%s" % mac_str
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("xen_add_vif: failed. vm_path('%s'), num('%s') => %s" % \
		(vm_path, num, errmsg))
	return "failed:%s" % errmsg

@exposed
def xen_set_vifs(vm_path, vif_cfg, is_append=False):
    """
    set vif to vm config file

    @param vif_cfg  of format "<mac>,<bridge>;<mac>,<bridge>", 
                    mac, bridge can be empty, if empty will not specify in that vif. 
                    e.g.  "00:15:60:C3:07:A1,xenbr0;00:15:60:C3:07:A2,xenbr2;,xenbr1"
    @param is_append    remove original vifs or not
    @return "success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(vm_path)

	l_vif_cfg = vif_cfg.split(";")
	if len(l_vif_cfg) > 8: 
	    #http://wiki.xensource.com/xenwiki/XenFaq#head-9896478cf65a16f43ab4fb066f74c0e0d67a16ac
	    raise Exception("cannot add more than 8 interfaces.")

        vm_cfg_file = join(vm_path, "vm.cfg")
	cfg = xen_unmarshall_cfg(vm_cfg_file)

        l_vif_cfg = [vif.split(",") for vif in l_vif_cfg if vif.strip()]
        l_vif_cfg = [(mac.strip(), bridge.strip()) for (mac, bridge) in l_vif_cfg]
	if "vif" not in cfg: 
            cfg["vif"] = []
        
        idx = 0
	for idx, (mac, bridge) in enumerate(l_vif_cfg):
            vif = {}
            if idx < len(cfg["vif"]) and not is_append:
                lst = [map(string.strip, attr.split("=")) 
                        for attr in cfg["vif"][idx].split(",")]
                vif = dict(lst)
            if mac: vif["mac"] = mac
	    if bridge: vif["bridge"] = bridge
            vif_val = ",".join([ "%s=%s" % (attr, value) for (attr, value) in vif.items()])
            if idx < len(cfg["vif"]) and not is_append:
	        cfg["vif"][idx] = vif_val
            else:
                cfg["vif"].append(vif_val)

        if not is_append: 
            del cfg["vif"][idx+1:] #remove additional vif
	xen_marshall_cfg(cfg, vm_cfg_file)

	logger.info("xen_set_vifs: success. vm('%s') set vif=%s" % \
	    (vm_path, repr(cfg["vif"])))
	return "success"
    except Exception, e:
        errmsg = fmt_err_msg(e)
	logger.error("xen_set_vifs: error. vm('%s') vif_cfg=%s =>%s" % \
		(vm_path, vif_cfg, errmsg))
	return "failed:%s" % errmsg

@exposed
def xen_get_vifs(vm_path):
    """
    get vif from vm config file

    @return     "<vif>=<mac>,<bridge>;<vif>=<mac>,<bridge>...", e.g.  
                "vif0=00:15:60:C3:07:A1,xenbr0;vif1=00:15:60:C3:07:A2,xenbr2;vif3=,xenbr1;vif4=,"
    """
    logger = logging.getLogger("ovs.query")
    try:
	chk_vm_path(vm_path)

        vm_cfg_file = join(vm_path, "vm.cfg")
	cfg = xen_unmarshall_cfg(vm_cfg_file)
        rs = []
	if "vif" not in cfg: cfg["vif"] = []        
        for idx, vif in enumerate(cfg["vif"]):
            d = {"mac":"", "bridge":""}
            pairs = [map(string.strip, part.split("=")) 
                            for part in vif.split(",") if part.strip()]
            d.update(dict(pairs))
            rs.append("vif%d=%s,%s" % (idx, d["mac"], d["bridge"]))
        rs = ";".join(rs)

	logger.info("xen_get_vifs: success. vm('%s') rs=%s" % \
	    (vm_path, rs))
	return "success:%s" % rs
    except Exception, e:
        errmsg = fmt_err_msg(e)
	logger.error("xen_get_vifs: error. vm('%s') =>%s" % \
		(vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def xen_clear_vif_mac(vm_path):
    """
    clear vif mac from vm configure file, used by copy vm, etc
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(vm_path)

        vm_cfg_file = join(vm_path, "vm.cfg")
	cfg = xen_unmarshall_cfg(vm_cfg_file)
        
	if "vif" in cfg:
            vifs = []
            for vif in cfg["vif"]:
                attrs = [attr.strip() for attr in vif.split(",") 
                        if not re.match(r"^mac\s*=", attr.strip())]
                vifs.append(",".join(attrs))
            cfg["vif"] = vifs

	xen_marshall_cfg(cfg, vm_cfg_file)

	logger.info("xen_clear_vif_mac: success. vm('%s')" % vm_path)
	return "success"
    except Exception, e:
        errmsg = fmt_err_msg(e)
	logger.error("xen_clear_vif_mac: error. vm('%s') =>%s" % \
		(vm_path, errmsg))
	return "failed:%s" % errmsg

#----------------------------------------<> vnc password <>----
@exposed
def xen_get_vm_vnc_password(vm_path):
    """
    get vm vnc passoword
    """
    logger = logging.getLogger("ovs.query")
    try:
	chk_vm_path(vm_path)

	vm_cfg_file = join(vm_path, "vm.cfg")
	cfg = xen_unmarshall_cfg(vm_cfg_file)

	rs = xen_get_vm_type(vm_path)
	if rs.startswith("success"):
	    type = parse_result(rs)["type"]
	    if "hvm" == type:
		#hvm
		if "vncpasswd" in cfg:
		    vnc_password = cfg["vncpasswd"]
		else:
		    raise Exception("can not get vnc password => type('%s')" % type)
	    elif "para" == type:
		#para
		if "vfb" in cfg:
		    vfb = cfg["vfb"]
		    vfb_lst = vfb[0].split(',')

		    vnc_password = None
		    for item in vfb_lst:
			if item.split('=')[0].strip() == "vncpasswd":
			    vnc_password = item.split('=')[1].strip()
			    break
		    if None == vnc_password:
			raise Exception("Can not get vnc password => type('%s')" % type)
		else:
		    raise Exception("Can not get vnc password => type('%s')" % type)
	    else:
		raise Exception("Error getting VM type => type('%s')" % type)
	else:
	    raise Exception("Error getting VM type")

	logger.info("xen_get_vm_vnc_password: success. vm_path('%s')" % vm_path)
	return "success:vnc_password=%s" % vnc_password
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("xen_get_vm_vnc_password: failed. vm_path('%s') => %s" % \
		(vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def xen_set_vm_vnc_password(vm_path, vnc_password):
    """
    set vm vnc password
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(vm_path)
        
        if re.sub(r"[0-9a-zA-Z_]", "", vnc_password) != "":
            raise Exception("only alphanumeric chars and '_' are allowed in vnc password!")

	vm_cfg_file = join(vm_path, "vm.cfg")
	cfg = xen_unmarshall_cfg(vm_cfg_file)
    
	rs = xen_get_vm_type(vm_path)
	if rs.startswith("success"):
	    type = parse_result(rs)["type"]
	    if "hvm" == type:
		#hvm
		cfg["vnc"] = 1
		cfg["vncunused"] = 1
		cfg["vnclisten"] = "0.0.0.0"
		cfg["vncconsole"] = 1
		cfg["vncpasswd"] = vnc_password
	    elif "para" == type:
		#para
		vnc_cfg = "type=vnc,vncunused=1,vnclisten=0.0.0.0,vncpasswd=%s" % vnc_password
		cfg["vfb"] = []
		cfg["vfb"].append(vnc_cfg)
	    else:
		raise Exception("Error getting VM type('%s')" % type)
	    xen_marshall_cfg(cfg, vm_cfg_file)
	else:
	    raise Exception("Error getting VM type")

	logger.info("xen_set_vm_vnc_password: success. vm_path('%s')" % vm_path)
	return "success"
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("xen_set_vm_vnc_password: failed. vm_path('%s') => %s" % \
		(vm_path, errmsg))
	return "failed:%s" % errmsg

#----------------------------------------<> detect vm type (hvm, para) <>----
@exposed
def xen_get_vm_type(vm_path):
    """
    get vm type
    """
    logger = logging.getLogger("ovs.query")
    try:
	chk_vm_path(vm_path)

	vm_cfg_file = join(vm_path, "vm.cfg")
	cfg = xen_unmarshall_cfg(vm_cfg_file)
	
	if ("kernel" in cfg and "hvm" in cfg["kernel"]) or \
		("builder" in cfg and cfg["builder"] == "hvm") or \
		("device_model" in cfg and "qemu" in cfg["device_model"]):
		    #hvm
		    type = "hvm"
	else:
	    #para
	    type = "para"
	
	logger.info("xen_get_vm_type: success. vm_path('%s') => type('%s')" % \
		(vm_path, type))
	return "success:type=%s" % type
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("xen_get_vm_type: failed. vm_path('%s') => %s" % \
		(vm_path, errmsg))
	return "failed:%s" % errmsg


